@@ -1,9 +1,9 @@
 '''version'''
-__version__ = '1.2.0'
+__version__ = '1.6.0'
 '''author'''
 __author__ = 'Zhenchao Jin'
 '''title'''
-__title__ = 'sssegmentation'
+__title__ = 'SSSegmentation'
 '''description'''
 __description__ = 'SSSegmentation: An Open Source Supervised Semantic Segmentation Toolbox Based on PyTorch'
 '''url'''
@@ -13,4 +13,4 @@ __email__ = 'charlesblwx@gmail.com'
 '''license'''
 __license__ = 'Apache License 2.0'
 '''copyright'''
-__copyright__ = 'Copyright 2020-2022 Zhenchao Jin'
\ No newline at end of file
+__copyright__ = 'Copyright 2020-2030 Zhenchao Jin'
\ No newline at end of file
