@@ -1,18 +1,18 @@
 '''initialize'''
 from .dropout import (
-    BuildDropout
+    DropoutBuilder, BuildDropout
 )
 from .activation import (
-    BuildActivation
+    ActivationBuilder, BuildActivation
 )
 from .normalization import (
-    BuildNormalization, constructnormcfg
+    NormalizationBuilder, BuildNormalization
 )
 from .misc import (
     Scale, L2Norm, makedivisible, truncnormal
 )
 from .transformer import (
-    FFN, MultiheadAttention, nchwtonlc, nlctonchw, PatchEmbed, PatchMerging, AdaptivePadding
+    FFN, MultiheadAttention, PatchEmbed, PatchMerging, AdaptivePadding, PositionEmbeddingSine, nchwtonlc, nlctonchw, nlc2nchw2nlc, nchw2nlc2nchw
 )
 from .convolution import (
     DynamicConv2d, AdptivePaddingConv2d, SqueezeExcitationConv2d, DepthwiseSeparableConv2d, InvertedResidual, InvertedResidualV3
