@@ -1,6 +1,6 @@
 '''
 Function:
-    Implementation of Swin
+    Implementation of SwinTransformer
 Author:
     Zhenchao Jin
 '''
@@ -12,13 +12,11 @@ import torch.nn.functional as F
 import torch.utils.model_zoo as model_zoo
 import torch.utils.checkpoint as checkpoint
 from collections import OrderedDict
-from .bricks import PatchEmbed as PatchEmbedBase
-from .bricks import PatchMerging as PatchMergingBase
-from .bricks import BuildNormalization, BuildDropout, FFN, constructnormcfg
+from .bricks import BuildNormalization, BuildDropout, FFN, PatchEmbed, PatchMerging
 
 
-'''model urls'''
-model_urls = {
+'''DEFAULT_MODEL_URLS'''
+DEFAULT_MODEL_URLS = {
     'swin_tiny_patch4_window7_224': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_tiny_patch4_window7_224.pth',
     'swin_small_patch4_window7_224': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_small_patch4_window7_224.pth',
     'swin_base_patch4_window12_384': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_base_patch4_window12_384.pth',
@@ -27,32 +25,44 @@ model_urls = {
     'swin_base_patch4_window7_224_22k': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_base_patch4_window7_224_22k.pth',
     'swin_large_patch4_window12_384_22k': 'https://github.com/SwinTransformer/storage/releases/download/v1.0.0/swin_large_patch4_window12_384_22k.pth',
 }
-
-
-'''PatchMerging'''
-class PatchMerging(PatchMergingBase):
-    def __init__(self, **kwargs):
-        super(PatchMerging, self).__init__(**kwargs)
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        if self.norm is None: return {}
-        return {'PatchMerging.norm': self.norm}
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        return {'PatchMerging.reduction': self.reduction}
-
-
-'''PatchEmbed'''
-class PatchEmbed(PatchEmbedBase):
-    def __init__(self, **kwargs):
-        super(PatchEmbed, self).__init__(**kwargs)
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        if self.norm is None: return {}
-        return {'PatchEmbed.norm': self.norm}
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        return {'PatchEmbed.projection': self.projection}
+'''AUTO_ASSERT_STRUCTURE_TYPES'''
+AUTO_ASSERT_STRUCTURE_TYPES = {
+    'swin_tiny_patch4_window7_224': {
+        'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 96, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4, 
+        'depths': [2, 2, 6, 2], 'num_heads': [3, 6, 12, 24], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True, 
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
+    'swin_small_patch4_window7_224': {
+        'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 96, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4,
+        'depths': [2, 2, 18, 2], 'num_heads': [3, 6, 12, 24], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
+    'swin_base_patch4_window12_384': {
+        'pretrain_img_size': 384, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 12, 'mlp_ratio': 4,
+        'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
+    'swin_base_patch4_window7_224': {
+        'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4,
+        'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
+    'swin_base_patch4_window12_384_22k': {
+        'pretrain_img_size': 384, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 12, 'mlp_ratio': 4,
+        'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
+    'swin_base_patch4_window7_224_22k': {
+        'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4,
+        'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
+    'swin_large_patch4_window12_384_22k': {
+        'pretrain_img_size': 384, 'in_channels': 3, 'embed_dims': 192, 'patch_size': 4, 'window_size': 12, 'mlp_ratio': 4,
+        'depths': [2, 2, 18, 2], 'num_heads': [6, 12, 24, 48], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
+        'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
+    },
+}
 
 
 '''WindowMSA'''
@@ -64,7 +74,7 @@ class WindowMSA(nn.Module):
         self.num_heads = num_heads
         head_embed_dims = embed_dims // num_heads
         self.scale = qk_scale or head_embed_dims**-0.5
-        # define a parameter table of relative position bias
+        # a parameter table of relative position bias
         self.relative_position_bias_table = nn.Parameter(torch.zeros((2 * window_size[0] - 1) * (2 * window_size[1] - 1), num_heads))
         Wh, Ww = self.window_size
         rel_index_coords = self.doublestepseq(2 * Ww - 1, Wh, 1, Ww)
@@ -76,12 +86,6 @@ class WindowMSA(nn.Module):
         self.proj = nn.Linear(embed_dims, embed_dims)
         self.proj_drop = nn.Dropout(proj_drop_rate)
         self.softmax = nn.Softmax(dim=-1)
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        return {'WindowMSA.relative_position_bias_table': self.relative_position_bias_table}
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        return {'WindowMSA.qkv': self.qkv, 'WindowMSA.proj': self.proj}
     '''forward'''
     def forward(self, x, mask=None):
         B, N, C = x.shape
@@ -89,9 +93,7 @@ class WindowMSA(nn.Module):
         q, k, v = qkv[0], qkv[1], qkv[2]
         q = q * self.scale
         attn = (q @ k.transpose(-2, -1))
-        relative_position_bias = self.relative_position_bias_table[
-            self.relative_position_index.view(-1)
-        ].view(self.window_size[0] * self.window_size[1], self.window_size[0] * self.window_size[1], -1)
+        relative_position_bias = self.relative_position_bias_table[self.relative_position_index.view(-1)].view(self.window_size[0] * self.window_size[1], self.window_size[0] * self.window_size[1], -1)
         relative_position_bias = relative_position_bias.permute(2, 0, 1).contiguous()
         attn = attn + relative_position_bias.unsqueeze(0)
         if mask is not None:
@@ -104,7 +106,7 @@ class WindowMSA(nn.Module):
         x = self.proj(x)
         x = self.proj_drop(x)
         return x
-    '''double step seq'''
+    '''doublestepseq'''
     @staticmethod
     def doublestepseq(step1, len1, step2, len2):
         seq1 = torch.arange(0, step1 * len1, step1)
@@ -120,29 +122,10 @@ class ShiftWindowMSA(nn.Module):
         self.shift_size = shift_size
         assert 0 <= self.shift_size < self.window_size
         self.w_msa = WindowMSA(
-            embed_dims=embed_dims,
-            num_heads=num_heads,
-            window_size=(window_size, window_size),
-            qkv_bias=qkv_bias,
-            qk_scale=qk_scale,
-            attn_drop_rate=attn_drop_rate,
-            proj_drop_rate=proj_drop_rate,
+            embed_dims=embed_dims, num_heads=num_heads, window_size=(window_size, window_size), qkv_bias=qkv_bias,
+            qk_scale=qk_scale, attn_drop_rate=attn_drop_rate, proj_drop_rate=proj_drop_rate,
         )
         self.drop = BuildDropout(dropout_cfg)
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        tmp_layers = self.w_msa.zerowdlayers()
-        layers = dict()
-        for key, value in tmp_layers.items():
-            layers[f'ShiftWindowMSA.{key}'] = value
-        return layers
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        tmp_layers = self.w_msa.nonzerowdlayers()
-        layers = dict()
-        for key, value in tmp_layers.items():
-            layers[f'ShiftWindowMSA.{key}'] = value
-        return layers
     '''forward'''
     def forward(self, query, hw_shape):
         B, L, C = query.shape
@@ -194,14 +177,14 @@ class ShiftWindowMSA(nn.Module):
         x = x.view(B, H * W, C)
         x = self.drop(x)
         return x
-    '''window reverse'''
+    '''windowreverse'''
     def windowreverse(self, windows, H, W):
         window_size = self.window_size
         B = int(windows.shape[0] / (H * W / window_size / window_size))
         x = windows.view(B, H // window_size, W // window_size, window_size, window_size, -1)
         x = x.permute(0, 1, 3, 2, 4, 5).contiguous().view(B, H, W, -1)
         return x
-    '''window partition'''
+    '''windowpartition'''
     def windowpartition(self, x):
         B, H, W, C = x.shape
         window_size = self.window_size
@@ -217,40 +200,18 @@ class SwinBlock(nn.Module):
                  attn_drop_rate=0., drop_path_rate=0., act_cfg=None, norm_cfg=None, use_checkpoint=False):
         super(SwinBlock, self).__init__()
         self.use_checkpoint = use_checkpoint
-        self.norm1 = BuildNormalization(constructnormcfg(placeholder=embed_dims, norm_cfg=norm_cfg))
+        self.norm1 = BuildNormalization(placeholder=embed_dims, norm_cfg=norm_cfg)
         self.attn = ShiftWindowMSA(
             embed_dims=embed_dims, num_heads=num_heads, window_size=window_size, shift_size=window_size // 2 if shift else 0,
             qkv_bias=qkv_bias, qk_scale=qk_scale, attn_drop_rate=attn_drop_rate, proj_drop_rate=drop_rate,
-            dropout_cfg={'type': 'droppath', 'drop_prob': drop_path_rate},
+            dropout_cfg={'type': 'DropPath', 'drop_prob': drop_path_rate},
         )
-        self.norm2 = BuildNormalization(constructnormcfg(placeholder=embed_dims, norm_cfg=norm_cfg))
+        self.norm2 = BuildNormalization(placeholder=embed_dims, norm_cfg=norm_cfg)
         self.ffn = FFN(
             embed_dims=embed_dims, feedforward_channels=feedforward_channels, num_fcs=2,
-            ffn_drop=drop_rate, dropout_cfg={'type': 'droppath', 'drop_prob': drop_path_rate},
+            ffn_drop=drop_rate, dropout_cfg={'type': 'DropPath', 'drop_prob': drop_path_rate},
             act_cfg=act_cfg, add_identity=True,
         )
-    '''layers with zero weight decay'''
-    def zerowdlayers(self, apply_wd_to_relative_position_bias_table=True):
-        # Note: our experiments find that do not set the weight decay of relative_position_bias_table as zero can achieve better segmentation performance
-        if apply_wd_to_relative_position_bias_table: return {'SwinBlock.norm1': self.norm1, 'SwinBlock.norm2': self.norm2}
-        layers = {'SwinBlock.norm1': self.norm1, 'SwinBlock.norm2': self.norm2}
-        for key, value in self.attn.zerowdlayers().items():
-            if 'relative_position_bias_table' in key:
-                layers[f'SwinBlock.{key}'] = nn.ParameterList([value])
-            else:
-                layers[f'SwinBlock.{key}'] = value
-        return layers
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self, apply_wd_to_relative_position_bias_table=True):
-        # Note: our experiments find that do not set the weight decay of relative_position_bias_table as zero can achieve better segmentation performance
-        if apply_wd_to_relative_position_bias_table: return {'SwinBlock.attn': self.attn, 'SwinBlock.ffn': self.ffn}
-        layers = {'SwinBlock.ffn': self.ffn}
-        for key, value in self.attn.nonzerowdlayers().items():
-            if 'relative_position_bias_table' in key:
-                layers[f'SwinBlock.{key}'] = nn.ParameterList([value])
-            else:
-                layers[f'SwinBlock.{key}'] = value
-        return layers
     '''forward'''
     def forward(self, x, hw_shape):
         def _forward(x):
@@ -284,38 +245,6 @@ class SwinBlockSequence(nn.Module):
             )
             self.blocks.append(block)
         self.downsample = downsample
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        layers = {}
-        for layer_idx, block in enumerate(self.blocks):
-            tmp_layers = block.zerowdlayers()
-            new_tmp_layers = {}
-            for key, value in tmp_layers.items():
-                new_tmp_layers[f"SwinBlockSequence.{layer_idx}_{key}"] = value
-            layers.update(new_tmp_layers)
-        if self.downsample is not None:
-            tmp_layers = self.downsample.zerowdlayers()
-            new_tmp_layers = {}
-            for key, value in tmp_layers.items():
-                new_tmp_layers[f"SwinBlockSequence.{key}"] = value
-            layers.update(new_tmp_layers)
-        return layers
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        layers = {}
-        for layer_idx, block in enumerate(self.blocks):
-            tmp_layers = block.nonzerowdlayers()
-            new_tmp_layers = {}
-            for key, value in tmp_layers.items():
-                new_tmp_layers[f"SwinBlockSequence.{layer_idx}_{key}"] = value
-            layers.update(new_tmp_layers)
-        if self.downsample is not None:
-            tmp_layers = self.downsample.nonzerowdlayers()
-            new_tmp_layers = {}
-            for key, value in tmp_layers.items():
-                new_tmp_layers[f"SwinBlockSequence.{key}"] = value
-            layers.update(new_tmp_layers)
-        return layers
     '''forward'''
     def forward(self, x, hw_shape):
         for block in self.blocks:
@@ -329,23 +258,44 @@ class SwinBlockSequence(nn.Module):
 
 '''SwinTransformer'''
 class SwinTransformer(nn.Module):
-    def __init__(self, pretrain_img_size=224, in_channels=3, embed_dims=96, patch_size=4, window_size=7, mlp_ratio=4, depths=(2, 2, 6, 2), num_heads=(3, 6, 12, 24),
+    def __init__(self, structure_type, pretrain_img_size=224, in_channels=3, embed_dims=96, patch_size=4, window_size=7, mlp_ratio=4, depths=(2, 2, 6, 2), num_heads=(3, 6, 12, 24),
                  strides=(4, 2, 2, 2), out_indices=(0, 1, 2, 3), qkv_bias=True, qk_scale=None, patch_norm=True, drop_rate=0., attn_drop_rate=0., drop_path_rate=0.1,
-                 use_abs_pos_embed=False, act_cfg=None, norm_cfg=None, use_checkpoint=False):
+                 use_abs_pos_embed=False, act_cfg={'type': 'GELU'}, norm_cfg={'type': 'LayerNorm'}, use_checkpoint=False, pretrained=True, pretrained_model_path=''):
         super(SwinTransformer, self).__init__()
-        if isinstance(pretrain_img_size, int):
-            pretrain_img_size = (pretrain_img_size, pretrain_img_size)
-        num_layers = len(depths)
+        # set attributes
+        self.structure_type = structure_type
+        self.pretrain_img_size = pretrain_img_size
+        self.in_channels = in_channels
+        self.embed_dims = embed_dims
+        self.patch_size = patch_size
+        self.window_size = window_size
+        self.mlp_ratio = mlp_ratio
+        self.depths = depths
+        self.num_heads = num_heads
+        self.strides = strides
         self.out_indices = out_indices
+        self.qkv_bias = qkv_bias
+        self.qk_scale = qk_scale
+        self.patch_norm = patch_norm
+        self.drop_rate = drop_rate
+        self.attn_drop_rate = attn_drop_rate
+        self.drop_path_rate = drop_path_rate
+        self.act_cfg = act_cfg
+        self.norm_cfg = norm_cfg
         self.use_abs_pos_embed = use_abs_pos_embed
+        self.use_checkpoint = use_checkpoint
+        self.pretrained = pretrained
+        self.pretrained_model_path = pretrained_model_path
+        # assert
         assert strides[0] == patch_size, 'Use non-overlapping patch embed.'
+        if structure_type in AUTO_ASSERT_STRUCTURE_TYPES:
+            for key, value in AUTO_ASSERT_STRUCTURE_TYPES[structure_type].items():
+                assert hasattr(self, key) and (getattr(self, key) == value)
+        if isinstance(pretrain_img_size, int): pretrain_img_size = (pretrain_img_size, pretrain_img_size)
+        self.pretrain_img_size = pretrain_img_size
+        # patch embedding
         self.patch_embed = PatchEmbed(
-            in_channels=in_channels,
-            embed_dims=embed_dims,
-            kernel_size=patch_size,
-            stride=strides[0],
-            padding='corner',
-            norm_cfg=norm_cfg if patch_norm else None,
+            in_channels=in_channels, embed_dims=embed_dims, kernel_size=patch_size, stride=strides[0], padding='corner', norm_cfg=norm_cfg if patch_norm else None,
         )
         if self.use_abs_pos_embed:
             patch_row = pretrain_img_size[0] // patch_size
@@ -358,13 +308,11 @@ class SwinTransformer(nn.Module):
         dpr = [x.item() for x in torch.linspace(0, drop_path_rate, total_depth)]
         self.stages = nn.ModuleList()
         in_channels = embed_dims
+        num_layers = len(depths)
         for i in range(num_layers):
             if i < num_layers - 1:
                 downsample = PatchMerging(
-                    in_channels=in_channels,
-                    out_channels=2 * in_channels,
-                    stride=strides[i + 1],
-                    norm_cfg=norm_cfg if patch_norm else None,
+                    in_channels=in_channels, out_channels=2 * in_channels, stride=strides[i + 1], norm_cfg=norm_cfg if patch_norm else None,
                 )
             else:
                 downsample = None
@@ -380,16 +328,19 @@ class SwinTransformer(nn.Module):
         self.num_features = [int(embed_dims * 2**i) for i in range(num_layers)]
         # add a norm layer for each output
         for i in out_indices:
-            layer = BuildNormalization(constructnormcfg(placeholder=self.num_features[i], norm_cfg=norm_cfg))
+            layer = BuildNormalization(placeholder=self.num_features[i], norm_cfg=norm_cfg)
             layer_name = f'norm{i}'
             self.add_module(layer_name, layer)
-    '''initialize backbone'''
-    def initweights(self, swin_type='swin_tiny_patch4_window7_224', pretrained_model_path=''):
+        # load pretrained weights
+        if pretrained:
+            self.loadpretrainedweights(structure_type, pretrained_model_path)
+    '''loadpretrainedweights'''
+    def loadpretrainedweights(self, structure_type='swin_tiny_patch4_window7_224', pretrained_model_path=''):
         # load
-        if pretrained_model_path:
+        if pretrained_model_path and os.path.exists(pretrained_model_path):
             checkpoint = torch.load(pretrained_model_path, map_location='cpu')
         else:
-            checkpoint = model_zoo.load_url(model_urls[swin_type], map_location='cpu')
+            checkpoint = model_zoo.load_url(DEFAULT_MODEL_URLS[structure_type], map_location='cpu')
         if 'state_dict' in checkpoint:
             state_dict = checkpoint['state_dict']
         elif 'model' in checkpoint:
@@ -429,7 +380,7 @@ class SwinTransformer(nn.Module):
                 state_dict[table_key] = table_pretrained_resized.view(nH2, L2).permute(1, 0).contiguous()
         # load state_dict
         self.load_state_dict(state_dict, strict=False)
-    '''swin convert'''
+    '''swinconvert'''
     @staticmethod
     def swinconvert(ckpt):
         new_ckpt = OrderedDict()
@@ -472,40 +423,6 @@ class SwinTransformer(nn.Module):
                 new_k = k
             new_ckpt[new_k] = new_v
         return new_ckpt
-    '''layers zero weight decay'''
-    def zerowdlayers(self):
-        layers = {}
-        if hasattr(self, 'absolute_pos_embed'):
-            layers['SwinTransformer.absolute_pos_embed'] = self.absolute_pos_embed
-        tmp_layers = self.patch_embed.zerowdlayers()
-        new_tmp_layers = {}
-        for key, value in tmp_layers.items():
-            new_tmp_layers[f'SwinTransformer.{key}'] = value
-        layers.update(new_tmp_layers)
-        for layer_idx, stage in enumerate(self.stages):
-            tmp_layers = stage.zerowdlayers()
-            new_tmp_layers = {}
-            for key, value in tmp_layers.items():
-                new_tmp_layers[f"SwinTransformer.{layer_idx}_{key}"] = value
-            layers.update(new_tmp_layers)
-        for idx in self.out_indices:
-            layers[f'SwinTransformer.norm{idx}'] = getattr(self, f'norm{idx}')
-        return layers
-    '''layers non zero weight decay'''
-    def nonzerowdlayers(self):
-        layers = {}
-        tmp_layers = self.patch_embed.nonzerowdlayers()
-        new_tmp_layers = {}
-        for key, value in tmp_layers.items():
-            new_tmp_layers[f'SwinTransformer.{key}'] = value
-        layers.update(new_tmp_layers)
-        for layer_idx, stage in enumerate(self.stages):
-            tmp_layers = stage.nonzerowdlayers()
-            new_tmp_layers = {}
-            for key, value in tmp_layers.items():
-                new_tmp_layers[f"SwinTransformer.{layer_idx}_{key}"] = value
-            layers.update(new_tmp_layers)
-        return layers
     '''forward'''
     def forward(self, x):
         x, hw_shape = self.patch_embed(x)
@@ -520,73 +437,4 @@ class SwinTransformer(nn.Module):
                 out = norm_layer(out)
                 out = out.view(-1, *out_hw_shape, self.num_features[i]).permute(0, 3, 1, 2).contiguous()
                 outs.append(out)
-        return outs
-
-
-'''BuildSwinTransformer'''
-def BuildSwinTransformer(swin_cfg):
-    # assert whether support
-    swin_type = swin_cfg.pop('type')
-    supported_swins = {
-        'swin_tiny_patch4_window7_224': {
-            'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 96, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4, 
-            'depths': [2, 2, 6, 2], 'num_heads': [3, 6, 12, 24], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True, 
-            'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
-        },
-        'swin_small_patch4_window7_224': {
-            'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 96, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4,
-            'depths': [2, 2, 18, 2], 'num_heads': [3, 6, 12, 24], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
-            'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
-        },
-        'swin_base_patch4_window12_384': {
-            'pretrain_img_size': 384, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 12, 'mlp_ratio': 4,
-            'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
-            'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
-        },
-        'swin_base_patch4_window7_224': {
-            'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4,
-            'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
-            'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
-        },
-        'swin_base_patch4_window12_384_22k': {
-            'pretrain_img_size': 384, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 12, 'mlp_ratio': 4,
-            'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
-            'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
-        },
-        'swin_base_patch4_window7_224_22k': {
-            'pretrain_img_size': 224, 'in_channels': 3, 'embed_dims': 128, 'patch_size': 4, 'window_size': 7, 'mlp_ratio': 4,
-            'depths': [2, 2, 18, 2], 'num_heads': [4, 8, 16, 32], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
-            'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
-        },
-        'swin_large_patch4_window12_384_22k': {
-            'pretrain_img_size': 384, 'in_channels': 3, 'embed_dims': 192, 'patch_size': 4, 'window_size': 12, 'mlp_ratio': 4,
-            'depths': [2, 2, 18, 2], 'num_heads': [6, 12, 24, 48], 'qkv_bias': True, 'qk_scale': None, 'patch_norm': True,
-            'drop_rate': 0., 'attn_drop_rate': 0., 'drop_path_rate': 0.3, 'use_abs_pos_embed': False,
-        },
-    }
-    assert swin_type in supported_swins, 'unsupport the swin_type %s' % swin_type
-    # parse cfg
-    default_cfg = {
-        'strides': (4, 2, 2, 2),
-        'out_indices': (0, 1, 2, 3),
-        'norm_cfg': {'type': 'layernorm'},
-        'act_cfg': {'type': 'gelu'},
-        'pretrained': True,
-        'pretrained_model_path': '',
-        'use_checkpoint': False,
-    }
-    default_cfg.update(supported_swins[swin_type])
-    for key, value in swin_cfg.items():
-        if key in default_cfg: 
-            default_cfg.update({key: value})
-    # obtain swin_cfg
-    swin_cfg = default_cfg.copy()
-    pretrained = swin_cfg.pop('pretrained')
-    pretrained_model_path = swin_cfg.pop('pretrained_model_path')
-    # obtain the instanced swin
-    model = SwinTransformer(**swin_cfg)
-    # load weights of pretrained model
-    if pretrained:
-        model.initweights(swin_type, pretrained_model_path)
-    # return the model
-    return model
\ No newline at end of file
+        return outs
\ No newline at end of file
