@@ -11,12 +11,11 @@ import torch.nn.functional as F
 import torch.utils.model_zoo as model_zoo
 from collections import OrderedDict
 from .mit import EfficientMultiheadAttention
-from .bricks import PatchEmbed as PatchEmbedBase
-from .bricks import BuildNormalization, FFN, BuildDropout, constructnormcfg
+from .bricks import BuildNormalization, FFN, BuildDropout, PatchEmbed
 
 
-'''model urls'''
-model_urls = {
+'''DEFAULT_MODEL_URLS'''
+DEFAULT_MODEL_URLS = {
     'pcpvt_small': 'https://github.com/SegmentationBLWX/modelstore/releases/download/ssseg_twins/pcpvt_small.pth',
     'pcpvt_base': 'https://github.com/SegmentationBLWX/modelstore/releases/download/ssseg_twins/pcpvt_base.pth',
     'pcpvt_large': 'https://github.com/SegmentationBLWX/modelstore/releases/download/ssseg_twins/pcpvt_large.pth',
@@ -24,36 +23,21 @@ model_urls = {
     'svt_base': 'https://github.com/SegmentationBLWX/modelstore/releases/download/ssseg_twins/alt_gvt_base.pth',
     'svt_large': 'https://github.com/SegmentationBLWX/modelstore/releases/download/ssseg_twins/alt_gvt_large.pth',
 }
-
-
-'''PatchEmbed'''
-class PatchEmbed(PatchEmbedBase):
-    def __init__(self, **kwargs):
-        super(PatchEmbed, self).__init__(**kwargs)
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        if self.norm is None: return {}
-        return {'PatchEmbed.norm': self.norm}
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        return {'PatchEmbed.projection': self.projection}
+'''AUTO_ASSERT_STRUCTURE_TYPES'''
+AUTO_ASSERT_STRUCTURE_TYPES = {
+    'pcpvt_small': {'depths': [3, 4, 6, 3], 'drop_path_rate': 0.2},
+    'pcpvt_base': {'depths': [3, 4, 18, 3], 'drop_path_rate': 0.3},
+    'pcpvt_large': {'depths': [3, 8, 27, 3], 'drop_path_rate': 0.3},
+    'svt_small': {'embed_dims': [64, 128, 256, 512], 'num_heads': [2, 4, 8, 16], 'mlp_ratios': [4, 4, 4, 4], 'depths': [2, 2, 10, 4], 'windiow_sizes': [7, 7, 7, 7], 'norm_after_stage': True, 'drop_path_rate': 0.2},
+    'svt_base': {'embed_dims': [96, 192, 384, 768], 'num_heads': [3, 6, 12, 24], 'mlp_ratios': [4, 4, 4, 4], 'depths': [2, 2, 18, 2], 'windiow_sizes': [7, 7, 7, 7], 'norm_after_stage': True, 'drop_path_rate': 0.2},
+    'svt_large': {'embed_dims': [128, 256, 512, 1024], 'num_heads': [4, 8, 16, 32], 'mlp_ratios': [4, 4, 4, 4], 'depths': [2, 2, 18, 2], 'windiow_sizes': [7, 7, 7, 7], 'norm_after_stage': True, 'drop_path_rate': 0.3},
+}
 
 
 '''GlobalSubsampledAttention'''
 class GlobalSubsampledAttention(EfficientMultiheadAttention):
     def __init__(self, embed_dims, num_heads, attn_drop=0., proj_drop=0., dropout_cfg=None, batch_first=True, qkv_bias=True, norm_cfg=None, sr_ratio=1):
         super(GlobalSubsampledAttention, self).__init__(embed_dims, num_heads, attn_drop, proj_drop, dropout_cfg, batch_first, qkv_bias, norm_cfg, sr_ratio)
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        if hasattr(self, 'norm'):
-            return {'GlobalSubsampledAttention.norm': self.norm}
-        return {}
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        layers = {'GlobalSubsampledAttention.attn': self.attn}
-        if hasattr(self, 'sr'):
-            layers.update({'GlobalSubsampledAttention.sr': self.sr})
-        return layers
 
 
 '''GSAEncoderLayer'''
@@ -61,13 +45,13 @@ class GSAEncoderLayer(nn.Module):
     def __init__(self, embed_dims, num_heads, feedforward_channels, drop_rate=0., attn_drop_rate=0., drop_path_rate=0., num_fcs=2, 
                  qkv_bias=True, act_cfg=None, norm_cfg=None, sr_ratio=1., dropout_cfg=None):
         super(GSAEncoderLayer, self).__init__()
-        if dropout_cfg is None: dropout_cfg = {'type': 'droppath', 'drop_prob': drop_path_rate}
-        self.norm1 = BuildNormalization(constructnormcfg(placeholder=embed_dims, norm_cfg=norm_cfg))
+        if dropout_cfg is None: dropout_cfg = {'type': 'DropPath', 'drop_prob': drop_path_rate}
+        self.norm1 = BuildNormalization(placeholder=embed_dims, norm_cfg=norm_cfg)
         self.attn = GlobalSubsampledAttention(
             embed_dims=embed_dims, num_heads=num_heads, attn_drop=attn_drop_rate, proj_drop=drop_rate, 
             dropout_cfg=dropout_cfg, qkv_bias=qkv_bias, norm_cfg=norm_cfg, sr_ratio=sr_ratio
         )
-        self.norm2 = BuildNormalization(constructnormcfg(placeholder=embed_dims, norm_cfg=norm_cfg))
+        self.norm2 = BuildNormalization(placeholder=embed_dims, norm_cfg=norm_cfg)
         self.ffn = FFN(
             embed_dims=embed_dims, feedforward_channels=feedforward_channels, num_fcs=num_fcs, ffn_drop=drop_rate,
             dropout_cfg=dropout_cfg, act_cfg=act_cfg, add_identity=False,
@@ -78,18 +62,6 @@ class GSAEncoderLayer(nn.Module):
         x = x + self.drop_path(self.attn(self.norm1(x), hw_shape, identity=0.))
         x = x + self.drop_path(self.ffn(self.norm2(x)))
         return x
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        layers = {'GSAEncoderLayer.norm1': self.norm1, 'GSAEncoderLayer.norm2': self.norm2}
-        for key, value in self.attn.zerowdlayers().items():
-            layers['GSAEncoderLayer.' + key] = value
-        return layers
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        layers = {'GSAEncoderLayer.ffn': self.ffn}
-        for key, value in self.attn.nonzerowdlayers().items():
-            layers['GSAEncoderLayer.' + key] = value
-        return layers
 
 
 '''LocallyGroupedSelfAttention'''
@@ -145,12 +117,6 @@ class LocallyGroupedSelfAttention(nn.Module):
         x = self.proj(x)
         x = self.proj_drop(x)
         return x
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        return {}
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        return {'LocallyGroupedSelfAttention.qkv': self.qkv, 'LocallyGroupedSelfAttention.proj': self.proj}
 
 
 '''LSAEncoderLayer'''
@@ -158,10 +124,10 @@ class LSAEncoderLayer(nn.Module):
     def __init__(self, embed_dims, num_heads, feedforward_channels, drop_rate=0., attn_drop_rate=0., drop_path_rate=0., 
                  num_fcs=2, qkv_bias=True, qk_scale=None, act_cfg=None, norm_cfg=None, window_size=1, dropout_cfg=None):
         super(LSAEncoderLayer, self).__init__()
-        if dropout_cfg is None: dropout_cfg = {'type': 'droppath', 'drop_prob': drop_path_rate}
-        self.norm1 = BuildNormalization(constructnormcfg(placeholder=embed_dims, norm_cfg=norm_cfg))
+        if dropout_cfg is None: dropout_cfg = {'type': 'DropPath', 'drop_prob': drop_path_rate}
+        self.norm1 = BuildNormalization(placeholder=embed_dims, norm_cfg=norm_cfg)
         self.attn = LocallyGroupedSelfAttention(embed_dims, num_heads, qkv_bias, qk_scale, attn_drop_rate, drop_rate, window_size)
-        self.norm2 = BuildNormalization(constructnormcfg(placeholder=embed_dims, norm_cfg=norm_cfg))
+        self.norm2 = BuildNormalization(placeholder=embed_dims, norm_cfg=norm_cfg)
         self.ffn = FFN(
             embed_dims=embed_dims, feedforward_channels=feedforward_channels, num_fcs=num_fcs, ffn_drop=drop_rate,
             dropout_cfg=dropout_cfg, act_cfg=act_cfg, add_identity=False,
@@ -172,21 +138,9 @@ class LSAEncoderLayer(nn.Module):
         x = x + self.drop_path(self.attn(self.norm1(x), hw_shape))
         x = x + self.drop_path(self.ffn(self.norm2(x)))
         return x
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        layers = {'LSAEncoderLayer.norm1': self.norm1, 'LSAEncoderLayer.norm2': self.norm2}
-        for key, value in self.attn.zerowdlayers().items():
-            layers['LSAEncoderLayer.' + key] = value
-        return layers
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        layers = {'LSAEncoderLayer.ffn': self.ffn}
-        for key, value in self.attn.nonzerowdlayers().items():
-            layers['LSAEncoderLayer.' + key] = value
-        return layers
 
 
-'''The Conditional Position Encoding (CPE) module'''
+'''ConditionalPositionEncoding'''
 class ConditionalPositionEncoding(nn.Module):
     def __init__(self, in_channels, embed_dims=768, stride=1):
         super(ConditionalPositionEncoding, self).__init__()
@@ -202,24 +156,38 @@ class ConditionalPositionEncoding(nn.Module):
         else: x = self.proj(cnn_feat)
         x = x.flatten(2).transpose(1, 2)
         return x
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        return {}
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        return {'ConditionalPositionEncoding.proj': self.proj}
 
 
 '''Twins-PCPVT'''
 class PCPVT(nn.Module):
-    def __init__(self, in_channels=3, embed_dims=[64, 128, 256, 512], patch_sizes=[4, 2, 2, 2], strides=[4, 2, 2, 2], num_heads=[1, 2, 4, 8], mlp_ratios=[4, 4, 4, 4],
-                 out_indices=(0, 1, 2, 3), qkv_bias=False, drop_rate=0., attn_drop_rate=0., drop_path_rate=0., depths=[3, 4, 6, 3], sr_ratios=[8, 4, 2, 1],
-                 norm_after_stage=False, norm_cfg=None, act_cfg=None):
+    def __init__(self, structure_type, in_channels=3, embed_dims=[64, 128, 320, 512], patch_sizes=[4, 2, 2, 2], strides=[4, 2, 2, 2], num_heads=[1, 2, 5, 8], 
+                 mlp_ratios=[8, 8, 4, 4], out_indices=(0, 1, 2, 3), qkv_bias=True, drop_rate=0., attn_drop_rate=0., drop_path_rate=0., depths=[3, 4, 6, 3], 
+                 sr_ratios=[8, 4, 2, 1], norm_after_stage=False, norm_cfg={'type': 'LayerNorm'}, act_cfg={'type': 'GELU'}, pretrained=True, pretrained_model_path=''):
         super(PCPVT, self).__init__()
         # set attributes
-        self.depths = depths
+        self.structure_type = structure_type
+        self.in_channels = in_channels
+        self.embed_dims = embed_dims
+        self.patch_sizes = patch_sizes
+        self.strides = strides
+        self.num_heads = num_heads
+        self.mlp_ratios = mlp_ratios
         self.out_indices = out_indices
+        self.qkv_bias = qkv_bias
+        self.drop_rate = drop_rate
+        self.attn_drop_rate = attn_drop_rate
+        self.drop_path_rate = drop_path_rate
+        self.depths = depths
+        self.sr_ratios = sr_ratios
         self.norm_after_stage = norm_after_stage
+        self.norm_cfg = norm_cfg
+        self.act_cfg = act_cfg
+        self.pretrained = pretrained
+        self.pretrained_model_path = pretrained_model_path
+        # assert
+        if structure_type in AUTO_ASSERT_STRUCTURE_TYPES:
+            for key, value in AUTO_ASSERT_STRUCTURE_TYPES[structure_type].items():
+                assert hasattr(self, key) and (getattr(self, key) == value)
         # patch embed
         self.patch_embeds = nn.ModuleList()
         self.position_encoding_drops = nn.ModuleList()
@@ -248,7 +216,10 @@ class PCPVT(nn.Module):
         if self.norm_after_stage:
             self.norm_list = nn.ModuleList()
             for dim in embed_dims: 
-                self.norm_list.append(BuildNormalization(constructnormcfg(placeholder=dim, norm_cfg=norm_cfg)))
+                self.norm_list.append(BuildNormalization(placeholder=dim, norm_cfg=norm_cfg))
+        # load pretrained weights
+        if pretrained:
+            self.loadpretrainedweights(structure_type, pretrained_model_path)
     '''forward'''
     def forward(self, x):
         outputs, b = list(), x.shape[0]
@@ -264,48 +235,13 @@ class PCPVT(nn.Module):
             if i in self.out_indices:
                 outputs.append(x)
         return tuple(outputs)
-    '''layers with zero weight decay'''
-    def zerowdlayers(self):
-        layers = {}
-        if hasattr(self, 'norm_list'): layers['PCPVT.norm_list'] = self.norm_list
-        for layer_idx, layer in enumerate(self.layers):
-            for blk_idx, blk in enumerate(layer):
-                for key, value in blk.zerowdlayers().items():
-                    assert f'PCPVT.{key}.{layer_idx}.{blk_idx}' not in layers
-                    layers[f'PCPVT.{key}.{layer_idx}.{blk_idx}'] = value
-        for layer_idx, layer in enumerate(self.position_encodings):
-            for key, value in layer.zerowdlayers().items():
-                assert f'PCPVT.{key}.{layer_idx}' not in layers
-                layers[f'PCPVT.{key}.{layer_idx}'] = value
-        for layer_idx, layer in enumerate(self.patch_embeds):
-            for key, value in layer.zerowdlayers().items():
-                assert f'PCPVT.{key}.{layer_idx}' not in layers
-                layers[f'PCPVT.{key}.{layer_idx}'] = value
-        return layers
-    '''layers with non zero weight decay'''
-    def nonzerowdlayers(self):
-        layers = {}
-        for layer_idx, layer in enumerate(self.layers):
-            for blk_idx, blk in enumerate(layer):
-                for key, value in blk.nonzerowdlayers().items():
-                    assert f'PCPVT.{key}.{layer_idx}.{blk_idx}' not in layers
-                    layers[f'PCPVT.{key}.{layer_idx}.{blk_idx}'] = value
-        for layer_idx, layer in enumerate(self.position_encodings):
-            for key, value in layer.nonzerowdlayers().items():
-                assert f'PCPVT.{key}.{layer_idx}' not in layers
-                layers[f'PCPVT.{key}.{layer_idx}'] = value
-        for layer_idx, layer in enumerate(self.patch_embeds):
-            for key, value in layer.nonzerowdlayers().items():
-                assert f'PCPVT.{key}.{layer_idx}' not in layers
-                layers[f'PCPVT.{key}.{layer_idx}'] = value
-        return layers
-    '''initialize backbone'''
-    def initweights(self, twins_type='pcpvt_small', pretrained_model_path=''):
+    '''loadpretrainedweights'''
+    def loadpretrainedweights(self, structure_type='pcpvt_small', pretrained_model_path=''):
         # load
-        if pretrained_model_path:
+        if pretrained_model_path and os.path.exists(pretrained_model_path):
             checkpoint = torch.load(pretrained_model_path, map_location='cpu')
         else:
-            checkpoint = model_zoo.load_url(model_urls[twins_type], map_location='cpu')
+            checkpoint = model_zoo.load_url(DEFAULT_MODEL_URLS[structure_type], map_location='cpu')
         if 'state_dict' in checkpoint:
             state_dict = checkpoint['state_dict']
         elif 'model' in checkpoint:
@@ -313,12 +249,12 @@ class PCPVT(nn.Module):
         else:
             state_dict = checkpoint
         # be consistent
-        state_dict = self.twinsconvert(twins_type, state_dict)
+        state_dict = self.twinsconvert(structure_type, state_dict)
         # load state_dict
         self.load_state_dict(state_dict, strict=False)
-    '''twins convert'''
+    '''twinsconvert'''
     @staticmethod
-    def twinsconvert(twins_type, ckpt):
+    def twinsconvert(structure_type, ckpt):
         new_ckpt = OrderedDict()
         for k, v in list(ckpt.items()):
             new_v = v
@@ -334,7 +270,7 @@ class PCPVT(nn.Module):
                     new_k = k.replace('mlp.fc1', 'ffn.layers.0.0')
                 elif 'mlp.fc2' in k:
                     new_k = k.replace('mlp.fc2', 'ffn.layers.1')
-                elif twins_type.startswith('pcpvt'):
+                elif structure_type.startswith('pcpvt'):
                     if 'attn.proj.' in k: new_k = k.replace('proj.', 'attn.out_proj.')
                     else: new_k = k
                 else:
@@ -356,13 +292,15 @@ class PCPVT(nn.Module):
 
 '''Twins-SVT'''
 class SVT(PCPVT):
-    def __init__(self, in_channels=3, embed_dims=[64, 128, 256], patch_sizes=[4, 2, 2, 2], strides=[4, 2, 2, 2], num_heads=[1, 2, 4], mlp_ratios=[4, 4, 4],
-                 out_indices=(0, 1, 2, 3), qkv_bias=False, drop_rate=0., attn_drop_rate=0., drop_path_rate=0.2, depths=[4, 4, 4], sr_ratios=[4, 2, 1],
-                 windiow_sizes=[7, 7, 7], norm_after_stage=True, norm_cfg=None, act_cfg=None):
+    def __init__(self, structure_type, in_channels=3, embed_dims=[64, 128, 256], patch_sizes=[4, 2, 2, 2], strides=[4, 2, 2, 2], num_heads=[1, 2, 4], mlp_ratios=[4, 4, 4],
+                 out_indices=(0, 1, 2, 3), qkv_bias=True, drop_rate=0., attn_drop_rate=0., drop_path_rate=0.2, depths=[4, 4, 4], sr_ratios=[8, 4, 2, 1],
+                 windiow_sizes=[7, 7, 7], norm_after_stage=True, norm_cfg={'type': 'LayerNorm'}, act_cfg={'type': 'GELU'}, pretrained=True, pretrained_model_path=''):
+        self.windiow_sizes = windiow_sizes
         super(SVT, self).__init__(
             in_channels=in_channels, embed_dims=embed_dims, patch_sizes=patch_sizes, strides=strides, num_heads=num_heads, mlp_ratios=mlp_ratios,
             out_indices=out_indices, qkv_bias=qkv_bias, drop_rate=drop_rate, attn_drop_rate=attn_drop_rate, drop_path_rate=drop_path_rate,
-            depths=depths, sr_ratios=sr_ratios, norm_after_stage=norm_after_stage, norm_cfg=norm_cfg, act_cfg=act_cfg
+            depths=depths, sr_ratios=sr_ratios, norm_after_stage=norm_after_stage, norm_cfg=norm_cfg, act_cfg=act_cfg, pretrained=False, pretrained_model_path='',
+            structure_type=structure_type,
         )
         # transformer encoder, stochastic depth decay rule
         dpr = [x.item() for x in torch.linspace(0, drop_path_rate, sum(depths))]
@@ -374,80 +312,6 @@ class SVT(PCPVT):
                         drop_rate=drop_rate, attn_drop_rate=attn_drop_rate, drop_path_rate=dpr[sum(depths[:k])+i], num_fcs=2,
                         qkv_bias=qkv_bias, window_size=windiow_sizes[k], norm_cfg=norm_cfg, act_cfg=act_cfg
                     )
-
-
-'''BuildTwins'''
-def BuildTwins(twins_cfg):
-    # assert whether support
-    twins_type = twins_cfg.pop('type')
-    supported_twins = {
-        'pcpvt_small': [
-            PCPVT, {'depths': [3, 4, 6, 3], 'drop_path_rate': 0.2},
-        ],
-        'pcpvt_base': [
-            PCPVT, {'depths': [3, 4, 18, 3], 'drop_path_rate': 0.3},
-        ],
-        'pcpvt_large': [
-            PCPVT, {'depths': [3, 8, 27, 3], 'drop_path_rate': 0.3},
-        ],
-        'svt_small': [
-            SVT, {'embed_dims': [64, 128, 256, 512], 'num_heads': [2, 4, 8, 16], 'mlp_ratios': [4, 4, 4, 4], 'depths': [2, 2, 10, 4], 'windiow_sizes': [7, 7, 7, 7], 'norm_after_stage': True, 'drop_path_rate': 0.2},
-        ],
-        'svt_base': [
-            SVT, {'embed_dims': [96, 192, 384, 768], 'num_heads': [3, 6, 12, 24], 'mlp_ratios': [4, 4, 4, 4], 'depths': [2, 2, 18, 2], 'windiow_sizes': [7, 7, 7, 7], 'norm_after_stage': True, 'drop_path_rate': 0.2},
-        ],
-        'svt_large': [
-            SVT, {'embed_dims': [128, 256, 512, 1024], 'num_heads': [4, 8, 16, 32], 'mlp_ratios': [4, 4, 4, 4], 'depths': [2, 2, 18, 2], 'windiow_sizes': [7, 7, 7, 7], 'norm_after_stage': True, 'drop_path_rate': 0.3},
-        ],
-    }
-    assert twins_type in supported_twins, 'unspport the twins_type %s' % twins_type
-    # parse cfg
-    default_cfg = {
-        'pretrained': True,
-        'pretrained_model_path': '',
-    }
-    if twins_type.startswith('pcpvt'):
-        default_cfg.update({
-            'in_channels': 3,
-            'embed_dims': [64, 128, 320, 512],
-            'patch_sizes': [4, 2, 2, 2],
-            'strides': [4, 2, 2, 2],
-            'num_heads': [1, 2, 5, 8],
-            'mlp_ratios': [8, 8, 4, 4],
-            'out_indices': (0, 1, 2, 3),
-            'qkv_bias': True,
-            'drop_rate': 0.0,
-            'attn_drop_rate': 0.0,
-            'sr_ratios': [8, 4, 2, 1],
-            'norm_cfg': {'type': 'layernorm'},
-            'act_cfg': {'type': 'gelu'},
-            'norm_after_stage': False
-        })
-    else:
-        default_cfg.update({
-            'in_channels': 3,
-            'patch_sizes': [4, 2, 2, 2],
-            'strides': [4, 2, 2, 2],
-            'out_indices': (0, 1, 2, 3),
-            'qkv_bias': True,
-            'drop_rate': 0.0,
-            'attn_drop_rate': 0.0,
-            'sr_ratios': [8, 4, 2, 1],
-            'norm_cfg': {'type': 'layernorm'},
-            'act_cfg': {'type': 'gelu'},
-        })
-    default_cfg.update(supported_twins[twins_type][1])
-    for key, value in twins_cfg.items():
-        if key in default_cfg: 
-            default_cfg.update({key: value})
-    # obtain twins_cfg
-    twins_cfg = default_cfg.copy()
-    pretrained = twins_cfg.pop('pretrained')
-    pretrained_model_path = twins_cfg.pop('pretrained_model_path')
-    # obtain the instanced twins
-    model = supported_twins[twins_type][0](**twins_cfg)
-    # load weights of pretrained model
-    if pretrained:
-        model.initweights(twins_type, pretrained_model_path)
-    # return the model
-    return model
\ No newline at end of file
+        # load pretrained weights
+        if pretrained:
+            self.loadpretrainedweights(structure_type, pretrained_model_path)
\ No newline at end of file
