@@ -1,5 +1,7 @@
 '''initialize'''
 from .fpn import FPN
+from .ema import EMASegmentor
 from .base import BaseSegmentor
+from .misc import attrfetcher, attrjudger
 from .feature2pyramid import Feature2Pyramid
 from .selfattention import SelfAttentionBlock
\ No newline at end of file
