@@ -7,7 +7,7 @@ Author:
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from ...backbones import BuildActivation, BuildNormalization, constructnormcfg
+from ...backbones import BuildActivation, BuildNormalization
 
 
 '''SelfAttentionBlock'''
@@ -18,12 +18,8 @@ class SelfAttentionBlock(nn.Module):
         super(SelfAttentionBlock, self).__init__()
         # key project
         self.key_project = self.buildproject(
-            in_channels=key_in_channels,
-            out_channels=transform_channels,
-            num_convs=key_query_num_convs,
-            use_norm=key_query_norm,
-            norm_cfg=norm_cfg,
-            act_cfg=act_cfg,
+            in_channels=key_in_channels, out_channels=transform_channels, num_convs=key_query_num_convs,
+            use_norm=key_query_norm, norm_cfg=norm_cfg, act_cfg=act_cfg,
         )
         # query project
         if share_key_query:
@@ -31,32 +27,20 @@ class SelfAttentionBlock(nn.Module):
             self.query_project = self.key_project
         else:
             self.query_project = self.buildproject(
-                in_channels=query_in_channels,
-                out_channels=transform_channels,
-                num_convs=key_query_num_convs,
-                use_norm=key_query_norm,
-                norm_cfg=norm_cfg,
-                act_cfg=act_cfg,
+                in_channels=query_in_channels, out_channels=transform_channels, num_convs=key_query_num_convs,
+                use_norm=key_query_norm, norm_cfg=norm_cfg, act_cfg=act_cfg,
             )
         # value project
         self.value_project = self.buildproject(
-            in_channels=key_in_channels,
-            out_channels=transform_channels if with_out_project else out_channels,
-            num_convs=value_out_num_convs,
-            use_norm=value_out_norm,
-            norm_cfg=norm_cfg,
-            act_cfg=act_cfg,
+            in_channels=key_in_channels, out_channels=transform_channels if with_out_project else out_channels, num_convs=value_out_num_convs,
+            use_norm=value_out_norm, norm_cfg=norm_cfg, act_cfg=act_cfg,
         )
         # out project
         self.out_project = None
         if with_out_project:
             self.out_project = self.buildproject(
-                in_channels=transform_channels,
-                out_channels=out_channels,
-                num_convs=value_out_num_convs,
-                use_norm=value_out_norm,
-                norm_cfg=norm_cfg,
-                act_cfg=act_cfg,
+                in_channels=transform_channels, out_channels=out_channels, num_convs=value_out_num_convs,
+                use_norm=value_out_norm, norm_cfg=norm_cfg, act_cfg=act_cfg,
             )
         # downsample
         self.query_downsample = query_downsample
@@ -88,18 +72,18 @@ class SelfAttentionBlock(nn.Module):
         if self.out_project is not None:
             context = self.out_project(context)
         return context
-    '''build project'''
+    '''buildproject'''
     def buildproject(self, in_channels, out_channels, num_convs, use_norm, norm_cfg, act_cfg):
         if use_norm:
             convs = [nn.Sequential(
                 nn.Conv2d(in_channels, out_channels, kernel_size=1, stride=1, padding=0, bias=False),
-                BuildNormalization(constructnormcfg(placeholder=out_channels, norm_cfg=norm_cfg)),
+                BuildNormalization(placeholder=out_channels, norm_cfg=norm_cfg),
                 BuildActivation(act_cfg),
             )]
             for _ in range(num_convs - 1):
                 convs.append(nn.Sequential(
                     nn.Conv2d(out_channels, out_channels, kernel_size=1, stride=1, padding=0, bias=False),
-                    BuildNormalization(constructnormcfg(placeholder=out_channels, norm_cfg=norm_cfg)),
+                    BuildNormalization(placeholder=out_channels, norm_cfg=norm_cfg),
                     BuildActivation(act_cfg),
                 ))
         else:
