@@ -11,8 +11,9 @@ import torch.nn.functional as F
 import torch.distributed as dist
 from ..base import BaseSegmentor
 from ..pspnet import PyramidPoolingModule
-from ...backbones import BuildActivation, BuildNormalization, constructnormcfg
-from .transformers import Predictor, SetCriterion, Transformer, HungarianMatcher
+from ....utils import SSSegOutputStructure
+from ...backbones import BuildActivation, BuildNormalization
+from .transformers import Predictor, SetCriterion, HungarianMatcher
 
 
 '''MaskFormer'''
@@ -22,12 +23,8 @@ class MaskFormer(BaseSegmentor):
         align_corners, norm_cfg, act_cfg, head_cfg = self.align_corners, self.norm_cfg, self.act_cfg, cfg['head']
         # build pyramid pooling module
         ppm_cfg = {
-            'in_channels': head_cfg['in_channels_list'][-1],
-            'out_channels': head_cfg['feats_channels'],
-            'pool_scales': head_cfg['pool_scales'],
-            'align_corners': align_corners,
-            'norm_cfg': copy.deepcopy(norm_cfg),
-            'act_cfg': copy.deepcopy(act_cfg),
+            'in_channels': head_cfg['in_channels_list'][-1], 'out_channels': head_cfg['feats_channels'], 'pool_scales': head_cfg['pool_scales'],
+            'align_corners': align_corners, 'norm_cfg': copy.deepcopy(norm_cfg), 'act_cfg': copy.deepcopy(act_cfg),
         }
         self.ppm_net = PyramidPoolingModule(**ppm_cfg)
         # build lateral convs
@@ -37,7 +34,7 @@ class MaskFormer(BaseSegmentor):
         for in_channels in head_cfg['in_channels_list'][:-1]:
             self.lateral_convs.append(nn.Sequential(
                 nn.Conv2d(in_channels, head_cfg['feats_channels'], kernel_size=1, stride=1, padding=0, bias=False),
-                BuildNormalization(constructnormcfg(placeholder=head_cfg['feats_channels'], norm_cfg=norm_cfg)),
+                BuildNormalization(placeholder=head_cfg['feats_channels'], norm_cfg=norm_cfg),
                 BuildActivation(act_cfg_copy),
             ))
         # build fpn convs
@@ -45,7 +42,7 @@ class MaskFormer(BaseSegmentor):
         for in_channels in [head_cfg['feats_channels'], ] * len(self.lateral_convs):
             self.fpn_convs.append(nn.Sequential(
                 nn.Conv2d(in_channels, head_cfg['feats_channels'], kernel_size=3, stride=1, padding=1, bias=False),
-                BuildNormalization(constructnormcfg(placeholder=head_cfg['feats_channels'], norm_cfg=norm_cfg)),
+                BuildNormalization(placeholder=head_cfg['feats_channels'], norm_cfg=norm_cfg),
                 BuildActivation(act_cfg_copy),
             ))
         # build decoder
@@ -56,6 +53,7 @@ class MaskFormer(BaseSegmentor):
         head_cfg['predictor']['mask_dim'] = head_cfg['mask_feats_channels']
         head_cfg['predictor']['in_channels'] = head_cfg['in_channels_list'][-1]
         self.decoder_predictor = Predictor(**head_cfg['predictor'])
+        # build matcher and criterion
         matcher = HungarianMatcher(**head_cfg['matcher'])
         weight_dict = {'loss_ce': head_cfg['matcher']['cost_class'], 'loss_mask': head_cfg['matcher']['cost_mask'], 'loss_dice': head_cfg['matcher']['cost_dice']}
         if head_cfg['predictor']['deep_supervision']:
@@ -69,13 +67,11 @@ class MaskFormer(BaseSegmentor):
         self.setauxiliarydecoder(cfg['auxiliary'])
         # freeze normalization layer if necessary
         if cfg.get('is_freeze_norm', False): self.freezenormalization()
-        # layer names for training tricks
-        self.layer_names = ['backbone_net', 'ppm_net', 'lateral_convs', 'fpn_convs', 'decoder_mask', 'decoder_predictor']
     '''forward'''
-    def forward(self, x, targets=None, losses_cfg=None):
-        img_size = x.size(2), x.size(3)
+    def forward(self, data_meta):
+        img_size = data_meta.images.size(2), data_meta.images.size(3)
         # feed to backbone network
-        backbone_outputs = self.transforminputs(self.backbone_net(x), selected_indices=self.cfg['backbone'].get('selected_indices'))
+        backbone_outputs = self.transforminputs(self.backbone_net(data_meta.images), selected_indices=self.cfg['backbone'].get('selected_indices'))
         # feed to pyramid pooling module
         ppm_out = self.ppm_net(backbone_outputs[-1])
         # apply fpn
@@ -93,22 +89,26 @@ class MaskFormer(BaseSegmentor):
         mask_features = self.decoder_mask(fpn_out)
         predictions = self.decoder_predictor(backbone_outputs[-1], mask_features)
         # forward according to the mode
-        if self.mode == 'TRAIN':
-            losses_dict = self.criterion(predictions, targets)
+        ssseg_outputs = SSSegOutputStructure(mode=self.mode, auto_validate=False)
+        if self.mode in ['TRAIN', 'TRAIN_DEVELOP']:
+            losses_dict = self.criterion(predictions, data_meta.getannotations())
             for k in list(losses_dict.keys()):
-                if k in self.criterion.weight_dict: losses_dict[k] *= self.criterion.weight_dict[k]
-                else: losses_dict.pop(k)
-            loss, losses_log_dict = 0, {}
-            for key, value in losses_dict.items():
-                loss += value
-                if dist.is_available() and dist.is_initialized():
-                    value = value.data.clone()
-                    dist.all_reduce(value.div_(dist.get_world_size()))
+                if k in self.criterion.weight_dict:
+                    losses_dict[k] *= self.criterion.weight_dict[k]
                 else:
-                    value = torch.Tensor([value.item()]).type_as(ppm_out)
-                losses_log_dict[key] = value
-            losses_log_dict['total'] = sum(losses_log_dict.values())
-            return loss, losses_log_dict
+                    losses_dict.pop(k)
+            loss, losses_log_dict = 0, {}
+            for loss_key, loss_value in losses_dict.items():
+                loss_value = loss_value.mean()
+                loss = loss + loss_value
+                loss_value = loss_value.data.clone()
+                if dist.is_available() and dist.is_initialized():
+                    dist.all_reduce(loss_value.div_(dist.get_world_size()), op=dist.ReduceOp.SUM)
+                losses_log_dict[loss_key] = loss_value.item()
+            losses_log_dict.update({'loss_total': sum(losses_log_dict.values())})
+            ssseg_outputs.setvariable('loss', loss)
+            ssseg_outputs.setvariable('losses_log_dict', losses_log_dict)
+            if self.mode in ['TRAIN']: return ssseg_outputs
         mask_cls_results = predictions['pred_logits']
         mask_pred_results = predictions['pred_masks']
         mask_pred_results = F.interpolate(mask_pred_results, size=img_size, mode='bilinear', align_corners=self.align_corners)
@@ -118,5 +118,6 @@ class MaskFormer(BaseSegmentor):
             mask_pred = mask_pred.sigmoid()
             semseg = torch.einsum('qc,qhw->chw', mask_cls, mask_pred)
             predictions.append(semseg.unsqueeze(0))
-        predictions = torch.cat(predictions, dim=0)
-        return predictions
\ No newline at end of file
+        seg_logits = torch.cat(predictions, dim=0)
+        ssseg_outputs.setvariable('seg_logits', seg_logits)            
+        return ssseg_outputs
\ No newline at end of file
