@@ -9,8 +9,13 @@ import numpy as np
 import torch.nn as nn
 import torch.nn.functional as F
 from ..base import FPN, BaseSegmentor
-from mmcv.ops import point_sample as PointSample
-from ...backbones import BuildActivation, BuildNormalization, constructnormcfg
+from ...losses import calculatelosses
+from ....utils import SSSegOutputStructure
+from ...backbones import BuildActivation, BuildNormalization
+try:
+    from mmcv.ops import point_sample as PointSample
+except:
+    PointSample = None
 
 
 '''PointRend'''
@@ -20,11 +25,8 @@ class PointRend(BaseSegmentor):
         align_corners, norm_cfg, act_cfg, head_cfg = self.align_corners, self.norm_cfg, self.act_cfg, cfg['head']
         # build fpn
         self.fpn_neck = FPN(
-            in_channels_list=head_cfg['fpn_in_channels_list'],
-            out_channels=head_cfg['feats_channels'],
-            upsample_cfg=head_cfg['upsample_cfg'],
-            norm_cfg=norm_cfg,
-            act_cfg=act_cfg,
+            in_channels_list=head_cfg['fpn_in_channels_list'], out_channels=head_cfg['feats_channels'], upsample_cfg=head_cfg['upsample_cfg'],
+            norm_cfg=norm_cfg, act_cfg=act_cfg,
         )
         self.scale_heads, feature_stride_list = nn.ModuleList(), head_cfg['feature_stride_list']
         for i in range(len(feature_stride_list)):
@@ -33,13 +35,11 @@ class PointRend(BaseSegmentor):
             for k in range(head_length):
                 scale_head.append(nn.Sequential(
                     nn.Conv2d(head_cfg['feats_channels'] if k == 0 else head_cfg['scale_head_channels'], head_cfg['scale_head_channels'], kernel_size=3, stride=1, padding=1, bias=False),
-                    BuildNormalization(constructnormcfg(placeholder=head_cfg['scale_head_channels'], norm_cfg=norm_cfg)),
+                    BuildNormalization(placeholder=head_cfg['scale_head_channels'], norm_cfg=norm_cfg),
                     BuildActivation(act_cfg),
                 ))
                 if feature_stride_list[i] != feature_stride_list[0]:
-                    scale_head.append(
-                        nn.Upsample(scale_factor=2, mode='bilinear', align_corners=align_corners)
-                    )
+                    scale_head.append(nn.Upsample(scale_factor=2, mode='bilinear', align_corners=align_corners))
             self.scale_heads.append(nn.Sequential(*scale_head))
         # point rend
         self.num_fcs, self.coarse_pred_each_layer = head_cfg['num_fcs'], head_cfg['coarse_pred_each_layer']
@@ -49,7 +49,7 @@ class PointRend(BaseSegmentor):
         for k in range(self.num_fcs):
             fc = nn.Sequential(
                 nn.Conv1d(fc_in_channels, fc_channels, kernel_size=1, stride=1, padding=0, bias=False),
-                BuildNormalization(constructnormcfg(placeholder=fc_channels, norm_cfg=norm_cfg)),
+                BuildNormalization(placeholder=fc_channels, norm_cfg=norm_cfg),
                 BuildActivation(act_cfg),
             )
             self.fcs.append(fc)
@@ -65,72 +65,70 @@ class PointRend(BaseSegmentor):
         self.setauxiliarydecoder(cfg['auxiliary'])
         # freeze normalization layer if necessary
         if cfg.get('is_freeze_norm', False): self.freezenormalization()
-        # layer names for training tricks
-        self.layer_names = ['backbone_net', 'fpn_neck', 'scale_heads', 'fcs', 'decoder', 'auxiliary_decoder']
     '''forward'''
-    def forward(self, x, targets=None, losses_cfg=None):
-        img_size = x.size(2), x.size(3)
+    def forward(self, data_meta):
+        img_size = data_meta.images.size(2), data_meta.images.size(3)
         # feed to backbone network
-        backbone_outputs = self.transforminputs(self.backbone_net(x), selected_indices=self.cfg['backbone'].get('selected_indices'))
+        backbone_outputs = self.transforminputs(self.backbone_net(data_meta.images), selected_indices=self.cfg['backbone'].get('selected_indices'))
         # feed to fpn
         fpn_outs = self.fpn_neck(list(backbone_outputs))
         feats = self.scale_heads[0](fpn_outs[0])
         for i in range(1, len(self.cfg['head']['feature_stride_list'])):
             feats = feats + F.interpolate(self.scale_heads[i](fpn_outs[i]), size=feats.shape[2:], mode='bilinear', align_corners=self.align_corners)
         # feed to auxiliary decoder
-        predictions_aux = self.auxiliary_decoder(feats)
+        seg_logits_aux = self.auxiliary_decoder(feats)
         feats = fpn_outs[0]
-        # if mode is TRAIN
-        if self.mode == 'TRAIN':
+        # if mode is TRAIN or TRAIN_DEVELOP
+        ssseg_outputs = SSSegOutputStructure(mode=self.mode, auto_validate=False)
+        if self.mode in ['TRAIN', 'TRAIN_DEVELOP']:
             with torch.no_grad():
-                points = self.getpointstrain(predictions_aux, self.calculateuncertainty, cfg=self.cfg['head']['train'])
+                points = self.getpointstrain(seg_logits_aux, self.calculateuncertainty, cfg=self.cfg['head']['train'])
             fine_grained_point_feats = self.getfinegrainedpointfeats([feats], points)
-            coarse_point_feats = self.getcoarsepointfeats(predictions_aux, points)
-            outputs = torch.cat([fine_grained_point_feats, coarse_point_feats], dim=1)
+            coarse_point_feats = self.getcoarsepointfeats(seg_logits_aux, points)
+            feats_concat = torch.cat([fine_grained_point_feats, coarse_point_feats], dim=1)
             for fc in self.fcs:
-                outputs = fc(outputs)
+                feats_concat = fc(feats_concat)
                 if self.coarse_pred_each_layer:
-                    outputs = torch.cat([outputs, coarse_point_feats], dim=1)
-            predictions = self.decoder(outputs)
-            point_labels = PointSample(targets['segmentation'].unsqueeze(1).float(), points, mode='nearest', align_corners=self.align_corners)
+                    feats_concat = torch.cat([feats_concat, coarse_point_feats], dim=1)
+            seg_logits = self.decoder(feats_concat)
+            point_labels = PointSample(data_meta.getannotations()['seg_targets'].unsqueeze(1).float(), points, mode='nearest', align_corners=self.align_corners)
             point_labels = point_labels.squeeze(1).long()
-            targets['point_labels'] = point_labels
-            predictions_aux = F.interpolate(predictions_aux, size=img_size, mode='bilinear', align_corners=self.align_corners)
-            return self.calculatelosses(
-                predictions={'loss_cls': predictions, 'loss_aux': predictions_aux}, 
-                targets=targets,
-                losses_cfg=losses_cfg,
-                map_preds_to_tgts_dict={'loss_cls': 'point_labels', 'loss_aux': 'segmentation'}
+            annotations = data_meta.getannotations()
+            annotations['point_labels'] = point_labels
+            seg_logits_aux = F.interpolate(seg_logits_aux, size=img_size, mode='bilinear', align_corners=self.align_corners)
+            loss, losses_log_dict = calculatelosses(
+                predictions={'loss_cls': seg_logits, 'loss_aux': seg_logits_aux}, annotations=annotations, losses_cfg=self.cfg['losses'], preds_to_tgts_mapping={'loss_cls': 'point_labels', 'loss_aux': 'seg_targets'}, pixel_sampler=self.pixel_sampler
             )
+            ssseg_outputs.setvariable('loss', loss)
+            ssseg_outputs.setvariable('losses_log_dict', losses_log_dict)
+            if self.mode in ['TRAIN']: return ssseg_outputs
         # if mode is TEST
-        refined_seg_logits = predictions_aux.clone()
+        refined_seg_logits = seg_logits_aux.clone()
         for _ in range(self.cfg['head']['test']['subdivision_steps']):
             refined_seg_logits = F.interpolate(
-                input=refined_seg_logits, 
-                scale_factor=self.cfg['head']['test']['scale_factor'],
-                mode='bilinear',
-                align_corners=self.align_corners
+                input=refined_seg_logits, scale_factor=self.cfg['head']['test']['scale_factor'], mode='bilinear', align_corners=self.align_corners
             )
             batch_size, channels, height, width = refined_seg_logits.shape
             point_indices, points = self.getpointstest(refined_seg_logits, self.calculateuncertainty, cfg=self.cfg['head']['test'])
             fine_grained_point_feats = self.getfinegrainedpointfeats([feats], points)
-            coarse_point_feats = self.getcoarsepointfeats(predictions_aux, points)
-            outputs = torch.cat([fine_grained_point_feats, coarse_point_feats], dim=1)
+            coarse_point_feats = self.getcoarsepointfeats(seg_logits_aux, points)
+            feats_concat = torch.cat([fine_grained_point_feats, coarse_point_feats], dim=1)
             for fc in self.fcs:
-                outputs = fc(outputs)
+                feats_concat = fc(feats_concat)
                 if self.coarse_pred_each_layer:
-                    outputs = torch.cat([outputs, coarse_point_feats], dim=1)
-            predictions = self.decoder(outputs)
+                    feats_concat = torch.cat([feats_concat, coarse_point_feats], dim=1)
+            seg_logits = self.decoder(feats_concat)
             point_indices = point_indices.unsqueeze(1).expand(-1, channels, -1)
             refined_seg_logits = refined_seg_logits.reshape(batch_size, channels, height * width)
-            refined_seg_logits = refined_seg_logits.scatter_(2, point_indices, predictions)
+            refined_seg_logits = refined_seg_logits.scatter_(2, point_indices, seg_logits)
             refined_seg_logits = refined_seg_logits.view(batch_size, channels, height, width)
-        return refined_seg_logits
-    '''sample from coarse grained features'''
+        ssseg_outputs.setvariable('seg_logits', refined_seg_logits)
+        return ssseg_outputs
+    '''getcoarsepointfeats'''
     def getcoarsepointfeats(self, seg_logits, points):
         coarse_feats = PointSample(seg_logits, points, align_corners=self.align_corners)
         return coarse_feats
-    '''sample from fine grained features'''
+    '''getfinegrainedpointfeats'''
     def getfinegrainedpointfeats(self, x, points):
         fine_grained_feats_list = [PointSample(_, points, align_corners=self.align_corners) for _ in x]
         if len(fine_grained_feats_list) > 1:
@@ -138,12 +136,12 @@ class PointRend(BaseSegmentor):
         else:
             fine_grained_feats = fine_grained_feats_list[0]
         return fine_grained_feats
-    '''estimate uncertainty based on seg logits'''
+    '''calculateuncertainty'''
     @staticmethod
     def calculateuncertainty(seg_logits):
         top2_scores = torch.topk(seg_logits, k=2, dim=1)[0]
         return (top2_scores[:, 1] - top2_scores[:, 0]).unsqueeze(1)
-    '''sample points for training'''
+    '''getpointstrain'''
     def getpointstrain(self, seg_logits, uncertainty_func, cfg):
         # set attrs
         num_points = cfg['num_points']
@@ -160,14 +158,14 @@ class PointRend(BaseSegmentor):
         num_random_points = num_points - num_uncertain_points
         idx = torch.topk(point_uncertainties[:, 0, :], k=num_uncertain_points, dim=1)[1]
         shift = num_sampled * torch.arange(batch_size, dtype=torch.long, device=seg_logits.device)
-        idx += shift[:, None]
+        idx = idx + shift[:, None]
         point_coords = point_coords.view(-1, 2)[idx.view(-1), :].view(batch_size, num_uncertain_points, 2)
         if num_random_points > 0:
             rand_point_coords = torch.rand(batch_size, num_random_points, 2, device=seg_logits.device)
             point_coords = torch.cat((point_coords, rand_point_coords), dim=1)
         # return
         return point_coords
-    '''sample points for testing'''
+    '''getpointstest'''
     def getpointstest(self, seg_logits, uncertainty_func, cfg):
         num_points = cfg['subdivision_num_points']
         uncertainty_map = uncertainty_func(seg_logits)
